import { Http, I18n, Loader } from '@zeedhi/core';
import { Utilities } from '@/util/Utilities';
import { GenericValueController } from '@/controllers/GenericValueController';
import ChangeLabel from '@/helper/Labels';
// import { SvgMap } from '@zeedhi/common';

export class GraphicsController {
  public aFGeneralModal = Loader.getInstance('AFGeneralModalController');
  
  public aFBuyerModal = Loader.getInstance('AFBuyerModalController');
  
  public AFConfSupplierModal = Loader.getInstance('AFConfSupplierModalController');
  
  public AFDeliveryLastYearModal = Loader.getInstance('AFDeliveryLastYearModalController');
  
  public AverageTimeAFModal = Loader.getInstance('AverageTimeAFModalController');
    
  protected clickedArea: string = '';
  
  protected localData: { STARTDATE: string; ENDDATE: string; NMREGIAOGEO: string; CDOPERADOR: string; }[] = [];

  /**
 * Populate a pie chart component with data and update its options.
 *
 * This function is responsible for populating a pie chart component with data and updating its options.
 * The data is fetched and then is processed to extract labels and series values for the chart.
 * Finally, the component's series and chart options are updated based on the retrieved information.
 *
 * @param {Object} component - The specific pie chart component to be populated and updated.
 *
 * @access public
 * @author Jefferson Gouvea
 */
 
  public fillGraphicPie({ component }) {
    const props = component.options.props;
    const routeName = props.routeName;
    const seriesNameColumn = props.seriesNameColumn;
    const labelsNameColumn = props.labelsNameColumn;
    const formatting = new Utilities();
    const newLabels = [];
    const newSeries = [];
    const sendObject = {
      filter: [],
    };
    const genericTest = localStorage.getItem('GENERIC_VALUE');
    
    if (props.idOrigem) {
      sendObject.filter.push(
        {
          name: 'IDORIGEM',
          operator: '=',
          value: props.idOrigem,
        },
      );
    }
    
    if (genericTest === 'false' || genericTest === null) {
      Http.post(`/${routeName}`, sendObject).then((response) => {
        /* eslint no-console: ["error", { allow: ["warn", "error", "info", "log"] }] */
        const data = response.data.dataset[routeName];
        data.forEach((row) => {
          newLabels.push(ChangeLabel.getLabel(row[labelsNameColumn]));
          newSeries.push(row[seriesNameColumn]);
        });
        component.series = newSeries;
        const options = {
          labels: newLabels,
          yaxis: {
            labels: {
              formatter: (value) => (
                formatting.formatValue('float', value)
              ),
            },
          },
        };
        component.updateChart(options);
      });
    } else {
      const genericValues = new GenericValueController();
      const data = genericValues.valueGraphicPie[component.name].slice(0, 10);
      
      data.forEach((row) => {
        newLabels.push(ChangeLabel.getLabel(row[labelsNameColumn]));
        newSeries.push(row[seriesNameColumn]);
      });
      component.series = newSeries;
      const options = {
        labels: newLabels,
        yaxis: {
          labels: {
            formatter: (value) => (
              formatting.formatValue('float', value)
            ),
          },
        },
      };
      component.updateChart(options);
    }
  }
  
  /**
 * Populate a bar chart component with data and update its options.
 *
 * This function is responsible for populating a bar chart component with data and updating its options.
 * The data is fetched and then is processed to extract x-axis labels and series values for the chart.
 * Finally, the component's series and chart options are updated based on the retrieved information.
 * In the end of the function, if I have a graphic bar on the screen, it will immediatly change the color
 * of every draggable-hander component.
 *
 * @param {Object} component - The specific bar chart component to be populated and updated.
 *
 * @access public
 * @authot Jefferson Gouvea
 */
  public async fillGraphicBar({ component }) {
    const props = component.options.props;
    const routeName = props.routeName;
    const xaxisNameColumn = props.xaxisNameColumn;
    const seriesProps = props.series;
    const formatting = new Utilities();
    const changeColorDragComponent = new Utilities();
    const limit = props.limit ? props.limit : 10;
    const sendObject = {
      filter: [],
    };
    const genericValues = new GenericValueController();
    const genericTest = localStorage.getItem('GENERIC_VALUE');
    
    if (genericTest === 'false' || genericTest === null) {
      if (props.idOrigem) {
        sendObject.filter.push(
          {
            name: 'IDORIGEM',
            operator: '=',
            value: props.idOrigem,
          },
        );
      } else if (props.complete) {
        sendObject.filter.push(
          {
            name: 'COMPLETE',
            operator: '=',
            value: props.complete,
          },
        );
      }
      
      Http.post(`/${routeName}`, sendObject).then((response) => {
        const data = response.data.dataset[routeName].slice(0, limit);
        const xaxis = [];
        const newSeries = [];
        Object.keys(seriesProps).forEach((prop) => {
          newSeries.push({
            name: I18n.translate(seriesProps[prop].label),
            data: data.map((item) => item[prop]),
          });
        });
        
        data.forEach((row) => { xaxis.push(row[xaxisNameColumn].substring(0, 15)); });
        component.series = newSeries;
        if (routeName === 'aFDeliveryLastYear') {
          const options = {
            xaxis: {
              categories: xaxis,
            },
          };
          component.updateChart(options);
        } else {
          const options = {
            yaxis: {
              labels: {
                formatter: (value) => (
                  formatting.formatValue('float', value)
                ),
              },
            },
            xaxis: {
              categories: xaxis,
            },
          };
          component.updateChart(options);
        }
      });
    } else {
      const data = genericValues.valueGraphicBar[component.name].slice(0, limit);
      const xaxis = [];
      const newSeries = [];
      Object.keys(seriesProps).forEach((prop) => {
        newSeries.push({
          name: seriesProps[prop].label,
          data: data.map((item) => item[prop]),
        });
      });

      data.forEach((row) => { xaxis.push(row[xaxisNameColumn].substring(0, 15)); });
      component.series = newSeries;
      const options = {
        yaxis: {
          labels: {
            formatter: (value) => (
              formatting.formatValue('float', value)
            ),
          },
        },
        xaxis: {
          categories: xaxis,
        },
      };
      component.updateChart(options);
    }
    
    const zdDragHandler = document.querySelectorAll('.draggable-handle');
    let pathElement: any = 0;
    pathElement = zdDragHandler[0].querySelector('svg path');
    if (pathElement.getAttribute('fill')) {
      changeColorDragComponent.changeDragHandlerColor();
    }
  }
  
  /**
 * Populate a gauge chart component with data and update its options.
 *
 * This function is responsible for populating a gauge chart component with data and updating its options.
 * The data is fetched and then is processed to extract information for the gauge chart,
 * including title, subtext, and color configuration for the axis line. Finally, the component's chart
 * options are updated based on the retrieved information.
 *
 * @param {Object} component - The specific gauge chart component to be populated and updated.
 *
 * @access public
 * @author Jefferson Gouvea
 */
  public fillGraphicGauge({ component }) {
    const props = component.options.props;
    const routeName = props.routeName;
    const sendObject = {
      filter: [],
    };
    
    const genericValues = new GenericValueController();
    const genericTest = localStorage.getItem('GENERIC_VALUE');
    
    if (genericTest === 'false' || genericTest === null) {
      Http.post(`/${routeName}`, sendObject).then((response) => {
        const data = response.data.dataset[routeName][0];
        const formatedTitle = `${data.DSINDICADOR} - ${data.DATAAPURADO}`;
        const formatedSubtext = 'Meta';
        const options = {
          title: {
            text: I18n.translate(formatedTitle),
            subtext: `(${I18n.translate(formatedSubtext)}: ${data.VRLIMMETA}%)`,
          },
          series: [
            {
              axisLine: {
                lineStyle: {
                  color: [
                    [data.VRLIMMINIMO / 100, '#F44336'],
                    [data.VRLIMMETA / 100, '#F1C232'],
                    [1, '#6AA84F'],
                  ],
                },
              },
              data: [
                {
                  value: data.ENTREGA_TOTAL,
                },
              ],
            },
          ],
        };
        component.updateChart(options);
      });
    } else {
      const data = genericValues.valueGraphicGauge[routeName][0];
      
      const options = {
        series: [
          {
            axisLine: {
              lineStyle: {
                color: [
                  [data.VRLIMMINIMO / 100, '#F44336'],
                  [data.VRLIMMETA / 100, '#F1C232'],
                  [1, '#6AA84F'],
                ],
              },
            },
            data: [
              {
                value: data.ENTREGA_TOTAL,
              },
            ],
          },
        ],
      };
      component.updateChart(options);
    }
  }
  
  public fillGraphicPieDrill({ component }) {
    const props = component.options.props;
    const routeName = props.routeName;
    
    Http.get(`/${routeName}?requestType:FilterData`).then((data) => {
      const response = data.data.dataset[routeName];
      
      const fieldMapping = {
        LABEL: 'name',
        VALUE: 'value',
        IDCONFRECEBIMENTO: 'IDCONFRECEBIMENTO',
        CDOPERADOR: 'CDOPERADOR',
        REGIAO: 'REGIAO',
        NMESTADO: 'ESTADO',
      };
      
      const dataToChart = response.map((obj) => {
        const dataResponse = {};
        Object.keys(obj).forEach((key) => {
          if (fieldMapping[key]) {
            dataResponse[fieldMapping[key]] = obj[key];
          }
        });
        return dataResponse;
      });

      const options = {
        series: [
          {
            type: 'pie',
            data: [
              ...dataToChart,
            ],
          },
        ],
      };
      component.updateChart(options);
    });
  }
  
  public fillGraphicBarDrill({ component }: { component: any }) {
    const props = component.options.props;
    const routeName = props.routeName;
    const xaxisNameColumn = props.xaxisNameColumn;
    const formatting = new Utilities();
    
    Http.post(`/${routeName}?requestType=FilterData`).then((response) => {
      const data = response.data.dataset[routeName];
        
      const fieldMapping = {
        VRLIMMETA: 'Meta',
        TOTAL: 'Entregues',
        PARCIAL: 'Entrega Parcial',
        NAOENTREGUE: 'Não Entregues',
        DEVOLUCAO: 'Devolvidos',
        PENDENTE: 'Pendentes',
      };
      
      const dataToChart = {};
      const xaxis = [];
        
      data.forEach((row) => {
        const xaxisValue = row[xaxisNameColumn].substring(0, 15);
        xaxis.push(xaxisValue);

        Object.keys(fieldMapping).forEach((key) => {
          const seriesName = fieldMapping[key];
          if (!dataToChart[seriesName]) {
            dataToChart[seriesName] = [];
          }
          dataToChart[seriesName].push(row[key]);
        });
      });
        
      const series = Object.keys(dataToChart).map((name) => ({
        name,
        type: name === 'Meta' ? 'line' : 'bar',
        stack: name === 'Meta' ? undefined : 'total',
        data: dataToChart[name],
      }));
        
      const options = {
        yaxis: {
          labels: {
            formatter: (value) => (
              formatting.formatValue('float', value)
            ),
          },
        },
        xAxis: {
          data: xaxis,
        },
        series,
      };

      component.setOption(options);
    });
  }

  public drillDownPieChart({ component, param }) {
    const props = component.options.props;
    const routeName = props.routeName;
    const drillLevel = component.drillHistory.length;
    const seriesData = param.data;
    let title = '';
    
    if (drillLevel >= 3) {
      if (routeName === 'aFGeneral') {
        return this.aFGeneralModal.openAFGeneralModal({ component }, seriesData);
      } if (routeName === 'aFBuyer') {
        return this.aFBuyerModal.openAFBuyerModal({ component }, seriesData);
      }
      return this.AFConfSupplierModal.openAFConfSupplierModal({ component }, seriesData);
    }
    
    let queryString = `requestType=FilterData&level=${drillLevel}`;
    
    if (seriesData.IDCONFRECEBIMENTO !== undefined) {
      queryString += `&IDCONFRECEBIMENTO=${seriesData.IDCONFRECEBIMENTO}`;
    }
    
    if (seriesData.CDOPERADOR !== undefined) {
      queryString += `&operador=${seriesData.CDOPERADOR}`;
    }
    
    if (seriesData.REGIAO !== undefined) {
      queryString += `&regiao=${seriesData.REGIAO}`;
    }

    return Http.get(`/drillDownPieChart?${queryString}`).then((data) => {
      const response = data.data.dataset.drillDownPieChart;
      
      title = seriesData.name;
      component.options.fullTitle.text += `> ${title}`;
      
      const fieldMapping = {
        LABEL: 'name',
        VALUE: 'value',
        IDCONFRECEBIMENTO: 'IDCONFRECEBIMENTO',
        CDOPERADOR: 'CDOPERADOR',
        REGIAO: 'REGIAO',
        NMESTADO: 'ESTADO',
      };
    
      const dataToChart = response.map((obj) => {
        const dataResponse = {};
        Object.keys(obj).forEach((key) => {
          if (fieldMapping[key]) {
            dataResponse[fieldMapping[key]] = obj[key];
          }
        });
        return dataResponse;
      });
      
      const newOptions = {
        title: {
          text: title,
        },
        series: [
          {
            type: 'pie',
            data: [
              ...dataToChart,
            ],
          },
        ],
      };
      component.drillDown(newOptions);
    });
  }
  
  public getStartAndEndDate(monthYear: string): { startDate: string, endDate: string } {
    const [month, year] = monthYear.split('/').map(Number);
    
    const adjustedMonth = month - 1;
    
    const startDate = new Date(year, adjustedMonth, 1);
    const endDate = new Date(year, adjustedMonth + 1, 0);
    
    const startDateString = this.formatDate(startDate);
    const endDateString = this.formatDate(endDate);

    return {
      startDate: startDateString,
      endDate: endDateString,
    };
  }

  public formatDate(date: Date): string {
    const day = String(date.getDate()).padStart(2, '0');
    const month = String(date.getMonth() + 1).padStart(2, '0');
    const year = date.getFullYear();
    return `${day}/${month}/${year}`;
  }
  
  public drillDownStackedChart({ component, param }) {
    const seriesProps = component.options.series;
    const localData = component.options.localData;
    const drillLevel = component.drillHistory.length;
    let title = '';
    
    let queryString = `requestType=FilterData&level=${drillLevel}`;
    
    if (drillLevel >= 3) {
      return this.AFDeliveryLastYearModal.openAFDeliveryLastYearModal({ component, param });
    }
    
    if (drillLevel === 0) {
      const date = this.getStartAndEndDate(param.name);
      queryString += `&ENDDATE=${date.endDate}`;
      queryString += `&STARTDATE=${date.startDate}`;
    } else {
      if (localData[param.dataIndex].ENDDATE !== undefined) {
        queryString += `&ENDDATE=${localData[param.dataIndex].ENDDATE}`;
      }
    
      if (localData[param.dataIndex].STARTDATE !== undefined) {
        queryString += `&STARTDATE=${localData[param.dataIndex].STARTDATE}`;
      }
      
      if (localData[param.dataIndex].CDOPERADOR !== undefined) {
        queryString += `&CDOPERADOR=${localData[param.dataIndex].CDOPERADOR}`;
      }
      
      if (localData[param.dataIndex].NMREGIAOGEO !== undefined) {
        queryString += `&NMREGIAOGEO=${localData[param.dataIndex].NMREGIAOGEO}`;
      }
    }
  
    return Http.get(`/drillDownStackedChart?${queryString}`).then((response) => {
      const responseData = response.data.dataset.drillDownStackedChart;
      title = param.name;
      component.options.fullTitle.text += ` > ${title}`;
    
      const nameMapping = {
        Devolvidos: 'DEVOLUCAO',
        'Não Entregues': 'NAOENTREGUE',
        'Entrega Parcial': 'PARCIAL',
        Pendentes: 'PENDENTE',
        Entregues: 'TOTAL',
        Meta: 'META',
        STARTDATE: 'STARTDATE',
        ENDDATE: 'ENDDATE',
        NMREGIAOGEO: 'NMREGIAOGEO',
        CDOPERADOR: 'CDOPERADOR',
      };
    
      const mapDataToSeries = (data, seriesOptions) => {
        const categories = data.map((item) => item.LABEL);
      
        const seriesData = seriesOptions.map((series) => {
          const isMetaSeries = series.name === 'Meta';
          
          return {
            name: series.name,
            type: series.type,
            stack: series.stack,
            data: data.map((item) => {
              if (isMetaSeries) {
                return 99;
              }
              return item[nameMapping[series.name]];
            }),
          };
        });
    
        return {
          categories,
          seriesData,
        };
      };
    
      const { categories, seriesData } = mapDataToSeries(responseData, seriesProps);
      
      const dados = responseData.map((item) => ({
        STARTDATE: item[nameMapping.STARTDATE],
        ENDDATE: item[nameMapping.ENDDATE],
        NMREGIAOGEO: item[nameMapping.NMREGIAOGEO],
        CDOPERADOR: item[nameMapping.CDOPERADOR],
      }));
      
      component.options.localData = dados;
      
      const newOptions = {
        title: {
          text: title,
        },
        xAxis: {
          data: categories,
        },
        series: seriesData.map((series) => ({
          name: series.name,
          type: series.type,
          stack: series.stack,
          data: series.data,
        })),
      };
      component.drillDown(newOptions);
    });
  }
  
  /*
  public getDefaultDate(): { start, end } {
    const today = new Date();
    const start = new Date(today.getFullYear(), today.getMonth(), 1);
  
    const formatDate = (date): string => {
      const year = date.getFullYear();
      const month = (date.getMonth() + 1).toString().padStart(2, '0');
      const day = date.getDate().toString().padStart(2, '0');
      return `${day}/${month}/${year}`;
    };
  
    return {
      start: formatDate(start),
      end: formatDate(today),
    };
  }
    
  public fillGeoChart({ component }) {
    const defaultDate = this.getDefaultDate();
    
    let queryString = `requestType=FilterData`;
    queryString += `&STARTDATE=${defaultDate.start}`;
    queryString += `&ENDDATE=${defaultDate.end}`;
    
    return Http.post(`/getGeoChartData?${queryString}`).then(({ data }) => {
      component.svgContent = data.dataset.GeoChartData;
      console.warn(component.svgContent);
      component.updateSVGContainer();
    });
  }
  
  public updateGeoChart({ component, node }) {
    const defaultDate = node.label;
    const splitDates = (dateString): { start; end } => {
      const [start, end] = defaultDate.split(' a ');
      return { start, end };
    };
    
    const { start, end } = splitDates(defaultDate);
    
    let queryString = `requestType=FilterData`;
    queryString += `&STARTDATE=${start}`;
    queryString += `&ENDDATE=${end}`;
    
    return Http.post(`/getGeoChartData?${queryString}`).then(({ data }) => {
      component.svgContent = data.dataset.GeoChartData;
    });
  }

  public areaClick({ area, component }) {
    if (this.clickedArea === area.id) {
      return this.AverageTimeAFModal.openAverageTimeAFModal({ component }, area);
    }
    this.clickedArea = area.id;
  }
  */
  
  /**
 * Populate a data table component with formatted data and update it.
 *
 * This function is responsible for populating a data table component with formatted data and updating it.
 * The data is fetched and then is processed to format numerical values and update the table's data
 * source(grid modal for bar chart).
 *
 * @param {Object} component - The specific data table component to be populated and updated.
 *
 * @access public
 * @author Jefferson Gouvea
 */
  public tableData({ component }) {
    const formatting = new Utilities();
    const routeName = component.name;
    const sendObject = {
      filter: [],
    };
    let data;
    
    const genericValues = new GenericValueController();
    const genericTest = localStorage.getItem('GENERIC_VALUE');
    
    if (genericTest === 'false' || genericTest === null) {
      Http.post(`/${routeName}`, sendObject).then((response) => {
        data = response.data.dataset[routeName].map((obj) => {
          Object.keys(obj).forEach((key) => {
            if (typeof obj[key] === 'number') {
              obj[key] = formatting.formatValue('float', obj[key]);
            }
          });
        
          return obj;
        });
        component.datasource.updateData(data);
      });
    } else {
      data = genericValues.valueGraphicBar[routeName].map((obj) => {
        Object.keys(obj).forEach((key) => {
          if (typeof obj[key] === 'number') {
            obj[key] = formatting.formatValue('float', obj[key]);
          }
        });
      
        return obj;
      });
      
      component.datasource.updateData(data);
    }
  }

  /**
 * Populate a data table component with selected data and update it.
 *
 * This function is responsible for populating a data table component with selected data and updating it.
 * The selected data is filtered and fetched and then is processed to format numerical values and update
 * the table's data source(grid modal for pie chart).
 *
 * @param {Object} component - The specific data table component to be populated and updated with selected data.
 *
 * @access public
 * @authot Jefferson Gouvea
 */
  public tableForSelectedData({ component }) {
    const formatting = new Utilities();
    const routeName = component.name;
    const selectedData = component.datasource.filter.SELECTED;
    const sendObject = {
      filter: [
        {
          name: 'SELECTED',
          operator: '=',
          value: selectedData,
        },
      ],
    };
    let data;
    const genericTest = localStorage.getItem('GENERIC_VALUE');
    
    if (genericTest === 'false' || genericTest === null) {
      Http.post(`/${routeName}`, sendObject).then((response) => {
        data = response.data.dataset[routeName].map((obj) => {
          Object.keys(obj).forEach((key) => {
            if (typeof obj[key] === 'number') {
              obj[key] = formatting.formatValue('float', obj[key]);
            }
          });
        
          return obj;
        });
        component.datasource.setFilter([]);
        component.datasource.updateData(data);
      });
    } else {
      const genericValues = new GenericValueController();
      data = genericValues.valueGrid[component.name].map((obj) => {
        Object.keys(obj).forEach((key) => {
          if (typeof obj[key] === 'number') {
            obj[key] = formatting.formatValue('float', obj[key]);
          }
        });
      
        return obj;
      });
      
      component.datasource.updateData(data);
    }
  }
}
